#include "HSVConeWindow.h"
#include <iostream>
#include <math.h>
#include <windows.h>
#include <GL/gl.h>


HSVConeWindow::HSVConeWindow(int width, int height, char *title)
    :Fl_Gl_Window(width, height, title)
{
    theta = 0.0;
    phi = 3.142 * 0.5;
    range = 1.0;
}

HSVConeWindow::HSVConeWindow(int x, int y, int width, int height,
				 char *title)
    :Fl_Gl_Window(x, y, width, height, title)
{
    theta = 0.0;
    phi = 3.142 * 0.5;
    range = 1.0;
}


void
HSVConeWindow::HSV_To_RGB(float h_in, float s_in, float v_in,
                          float &r, float &g, float &b)
{
    float   h, f, p, q, t;
    int	    i;

    h_in = (float)fmod((float)h_in, 360.0f);
    if ( s_in > 1.0 )
	s_in = 1.0;
    if ( s_in < 0.0 )
	s_in = 0.0;
    if ( v_in > 1.0 )
	v_in = 1.0;
    if ( v_in < 0.0 )
	v_in = 0.0;

    if ( s_in == 0.0 )
	r = g = b = v_in;
    else
    {
	h = h_in / 60.0f;
	i = (int)floor(h);
	f = h - i;
	p = v_in * ( 1 - s_in );
	q = v_in * ( 1 - ( s_in * f ) );
	t = v_in * ( 1 - ( s_in * ( 1 - f ) ) );
	switch ( i )
	{
	  case 0:
	    r = v_in;
	    g = t;
	    b = p;
	    break;
	  case 1:
	    r = q;
	    g = v_in;
	    b = p;
	    break;
	  case 2:
	    r = p;
	    g = v_in;
	    b = t;
	    break;
	  case 3:
	    r = p;
	    g = q;
	    b = v_in;
	    break;
	  case 4:
	    r = t;
	    g = p;
	    b = v_in;
	    break;
	  case 5:
	    r = v_in;
	    g = p;
	    b = q;
	    break;
	}
    }
}


void
HSVConeWindow::Draw_Side_Face()
{
    int	    nh = 32;
    float   sh = 2.0 * 3.142 / (float)nh;
    int     nv = 4;
    float   sv = range / (float)nv;
    float   h;
    float   v;
    float   x, y;
    float   r, g, b;
    int     i, j;

    for ( j = 0, v = 0.0 ; j < nv ; j++, v += sv )
    {
	glBegin(GL_TRIANGLE_STRIP);
	for ( i = 0, h = 0 ; i <= nh ; i++, h += sh )
	{
	    HSV_To_RGB(h * 180.0f / 3.142f, v, v, r, g, b);
	    glColor3f(r, g, b);
	    x = v * (float)cos(h);
	    y = v * (float)sin(h);
	    glVertex3f(x, y, 2.0 * v);

	    HSV_To_RGB(h * 180.0f / 3.142f, v + sv, v + sv, r, g, b);
	    glColor3f(r, g, b);
	    x = ( v + sv ) * (float)cos(h);
	    y = ( v + sv ) * (float)sin(h);
	    glVertex3f(x, y, 2.0 * ( v + sv ));
	}
	glEnd();
    }
}


void
HSVConeWindow::Draw_Top_Face()
{
    int	    nh = 32;
    float   sh = 2.0 * 3.142 / (float)nh;
    int     ns = 4;
    float   ss = range / (float)ns;
    float   h;
    float   s;
    float   x, y;
    float   r, g, b;
    int     i, j;

    for ( j = 0, s = 0.0 ; j < ns ; j++, s += ss )
    {
	glBegin(GL_TRIANGLE_STRIP);
	for ( i = 0, h = 0 ; i <= nh ; i++, h += sh )
	{
	    HSV_To_RGB(h * 180.0f / 3.142f, s, range, r, g, b);
	    glColor3f(r, g, b);
	    x = s * (float)cos(h);
	    y = s * (float)sin(h);
	    glVertex3f(x, y, 2.0f * range);

	    HSV_To_RGB(h * 180.0f / 3.142f, s + ss, range, r, g, b);
	    glColor3f(r, g, b);
	    x = ( s + ss ) * (float)cos(h);
	    y = ( s + ss ) * (float)sin(h);
	    glVertex3f(x, y, 2.0f * range);
	}
	glEnd();
    }
}


void
HSVConeWindow::draw(void)
{
    if ( ! valid() )
    {
	glViewport(0, 0, w(), h());
	glLoadIdentity();
	glOrtho(-2.0, 2.0, -2.0, 2.0, -2.0, 2.0);
	glClearColor(0.5, 0.5, 0.5, 1.0);
	glEnable(GL_DEPTH_TEST);
    }

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();

    glRotated((phi * 180.0 / 3.142) - 180.0, 1.0, 0.0, 0.0);
    glRotated(theta * 180.0 / 3.142, 0.0, 0.0, 1.0);
    glTranslatef(0.0, 0.0, -1.0);

    Draw_Top_Face();
    Draw_Side_Face();

    glPopMatrix();

}


void
HSVConeWindow::Button_Down(void)
{
    start_x = Fl::event_x();
    start_y = Fl::event_y();
    if ( start_y == 0 ) start_y++;
    if ( start_y == h() ) start_y--;
    scale_to_top = (3.142 - phi) / (double)start_y;
    scale_to_bottom = phi / ((double)h() - start_y);
    scale_horiz = 2 * 3.142 / (double)w();
    init_theta = theta;
    init_phi = phi;
}


void
HSVConeWindow::Button_Drag(void)
{
    int	    x = Fl::event_x();
    int     y = Fl::event_y();

    if ( y >= h() )
        y = h() - 1;
    if ( y <= 0 ) y = 1;

    theta = init_theta - (start_x - x) * scale_horiz;
    phi = init_phi
        - (start_y - y ) * ((y < start_y) ? scale_to_top : scale_to_bottom);

    invalidate();
    Fl::redraw();
}


int
HSVConeWindow::handle(int e)
{
    switch ( e )
    {
      case FL_PUSH:
	Button_Down();
	return 1;
      case FL_DRAG:
      	Button_Drag();
	return 1;
    }

    return Fl_Window::handle(e);
}


